/*##############################################################################

nIP - Stack

File        : dns.h

Description : Domain Name System

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

##############################################################################*/

#ifndef _DNS_H_
 #define _DNS_H_


#define NIP_DNS_QR_OFFSET  7 /**< header QR field    , see rfc 1035 */
#define NIP_DNS_OP_OFFSET  3 /**< header Opcode field, see rfc 1035 */
#define NIP_DNS_AA_OFFSET  2 /**< header AA field    , see rfc 1035 */
#define NIP_DNS_TC_OFFSET  1 /**< header TC field    , see rfc 1035 */
#define NIP_DNS_RD_OFFSET  0 /**< header RD field    , see rfc 1035 */

#define NIP_DNS_RA_OFFSET  7 /**< header RA field    , see rfc 1035 */
#define NIP_DNS_Z_OFFSET   6 /**< header Z  field    , see rfc 2535 */
#define NIP_DNS_AD_OFFSET  5 /**< header AD field    , see rfc 2535 */
#define NIP_DNS_CD_OFFSET  4 /**< header CD field    , see rfc 2535 */
#define NIP_DNS_RC_OFFSET  0 /**< header RCODE field , see rfc 1035 */


#define NIP_DNS_QR_MASK    0x80
#define NIP_DNS_OP_MASK    0x78
#define NIP_DNS_AA_MASK    0x04
#define NIP_DNS_TC_MASK    0x02
#define NIP_DNS_RD_MASK    0x01

#define NIP_DNS_RA_MASK    0x80
#define NIP_DNS_Z_MASK     0x40
#define NIP_DNS_AD_MASK    0x20
#define NIP_DNS_CD_MASK    0x10
#define NIP_DNS_RC_MASK    0x07

#define NIP_DNS_CF_FLAG    0x80  /**< cache flush flag */

#define NIP_DNS_HSIZE      sizeof( struct nip_dns_header )

#define NIP_DNS_CLASS_IN   1

struct nip_dns_string
{
	uint8_t len;  /** string length */
	uint8_t s[0]; /** string */
}
#ifndef DOXYGEN
__attribute__((packed))
#endif
;

struct nip_dns_header
{
	uint16_t id;                 /**< query ID */
	uint8_t  qr_opcode_aa_tc_rd; /**< QR, Opcode, AA, TC, and RD fields */
	uint8_t  ra_z_ad_cd_rcode;   /**< RA, Z, AD, CD, and RCODE fields */
	uint16_t qa_count[4];        /**< 0: number of entries in question section
	                              * 1: number of resourec records in answer section
	                              * 2: number of name server resource records in
	                              *    the authority records section
	                              * 3: number of resource records in additional
	                              *    records section */

}
#ifndef DOXYGEN
__attribute__((packed))
#endif
;

#endif
