/*##############################################################################

nIP - nano IP stack

File        : ports.h

Description : Defines several standard-ports.

Copyright notice:

Copyright (C) 2006 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

##############################################################################*/


#ifndef _PORTS_H_
#define _PORTS_H_

// include definition of ENDIANNESS
#include NIP_ARCH_FILE(endianness.h)

// The predefined port numbers will be stored depending on the system's byte
// order. That will spare future conversions between host and network byte order.

#if ( ENDIANNESS == BIG_ENDIAN )

/** \name TCP Ports */
//@{
#define NIP_TCP_PORT_HTTP        0x0050  //!< decimal 80
#define NIP_TCP_PORT_TELNET      0x0017  //!< decimal 23

//@}

/** \name UDP Ports */
//@{
#define NIP_UDP_PORT_DHCPCLIENT  0x0044  //!< decimal 68
#define NIP_UDP_PORT_DHCPSERVER  0x0043  //!< decimal 67
#define NIP_UDP_PORT_NTP         0x0080  //!< decimal 123
#define NIP_UDP_PORT_MDNS        0x14E9  //!< decimal 5353
//@}

#else  // ENDIANNESS

/** \name TCP Ports */
//@{
#define NIP_TCP_PORT_HTTP        0x5000  //!< decimal 80
#define NIP_TCP_PORT_TELNET      0x1700  //!< decimal 23

//@}

/** \name UDP Ports */
//@{
#define NIP_UDP_PORT_DHCPCLIENT  0x4400  //!< decimal 68
#define NIP_UDP_PORT_DHCPSERVER  0x4300  //!< decimal 67
#define NIP_UDP_PORT_NTP         0x8000  //!< decimal 123
#define NIP_UDP_PORT_MDNS        0xE914  //!< decimal 5353
//@}

#endif // ENDIANNESS

#endif //_PORTS_H_
