/*##############################################################################

nIP - nano IP stack

File        : autoip.h

Description : IPv4 Link-Local implementation (according to RFC 3927)

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

##############################################################################*/

#ifndef _NIP_AUTOIP_H
 #define _NIP_AUTOIP_H

#include "nip_init.h"
#include "net_if.h"
#include "net/arp.h"

#define NIP_AUTOIP_PROBE_WAIT    1 /**< max. interval to wait before sending probe packets */
#define NIP_AUTOIP_PROBE_NUM     3 /**< number of probes packets to send */
#define NIP_AUTOIP_PROBE_MIN     1 /**< min. seconds to wait in between two probe packets */
#define NIP_AUTOIP_PROBE_MAX     2 /**< max. seconds to wait in between two probe packets */
#define NIP_AUTOIP_ANNOUNCE_WAIT 2 /**< seconds to wait after last probe packet is sent*/
#define NIP_AUTOIP_ANNOUNCE_NUM  2 /**< number of announcements to send */
#define NIP_AUTOIP_ANNOUNCE_INTERVAL 2 /**< seconds to wait inbetween announcements */
#define NIP_AUTOIP_MAX_CONFLICTS 10 /**< number of conflicts after which to limit probe rate */
#define NIP_AUTOIP_RATE_LIMIT_INTERVAL 60 /**< rate at which try new addresses after MAX_CONFLICTS conflicts */
#define NIP_AUTOIP_DEFEND_INTERVAL   10 /**< minimum interval between defensive ARPs */


typedef enum
{
	NIP_AUTOIP_STAT_DISABLED = 0,
	NIP_AUTOIP_STAT_INIT,
	NIP_AUTOIP_STAT_PROBE,
	NIP_AUTOIP_STAT_ANNOUNCE,
	NIP_AUTOIP_STAT_UP,
	NIP_AUTOIP_STAT_CONFLICT
}
#ifndef DOXYGEN
__attribute__((packed))
#endif
nip_autoip_state_t;

struct nip_autoip_conf
{
	nip_autoip_state_t state;
	uint8_t            addr[4]; /**< IPv4 address to probe for */
	nip_time_t         t;  /**< timeout variable for current state */
	uint8_t            cnt;/**< number of probes/announcements that have been sent */
	uint8_t            conflict_cnt;
}
#ifndef DOXYGEN
__attribute__((packed))
#endif
;

#if NIP_NET_IF_COUNT > 1
extern struct nip_autoip_conf nip_autoip_conf[];
#else
extern struct nip_autoip_conf nip_autoip_conf;
#endif
void nip_autoip_start( nip_net_if_id_t net_if );
void nip_autoip_disp_check( void );
void nip_autoip_arpcheck( nip_net_if_id_t if_id, struct ARP_PACKET *arp );

#endif /* _NIP_AUTOIP_H */
