/*##############################################################################

nIP - nano IP stack

File        : nip_init.c

Description : Initialize NIP Internetstack

Copyright notice:

Copyright (C) 2007 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

##############################################################################*/

#include "nip_init.h"
#include "net/net_if.h"
#include "mem.h"
#include "nip_error.h"
#include "app/mdns.h"

extern nip_net_if_t nip_net_if_list[];

/// global variable used by nIP functions to post additional error codes.
nip_error_t nip_error;

/** Initialize NIP Internetstack
 */
void nip_init()
{
	nip_net_if_id_t net_if_id = 0;

	// clear network interfaces
	#if (NIP_NET_ENABLE==1)
	do
	{
		nip_net_if_list[ net_if_id ].id = NIP_NET_NO_IF;

		/** \todo check if doing interface transmission memory allocation at this
		 * point makes more sense than doing it whenever the transmission is used.
		 * PRO:  no need to initialize before usage -> saves program memory
		 * CON: each allocated block eats up sizeof( nip_mem_block_t ) even when
		 *      not in use.
		 * There should be a Switch to decide which way to go.
		 */

	} while ( ++net_if_id < NIP_NET_IF_COUNT );
	#endif

	#if (NIP_MEM_ENABLE==1)
	// initialize dynamic memory
	nip_mem_init();
	#endif

	#if (NIP_MDNS_ENABLE == 1)
	// initialize MDNS buffers
	nip_mdns_init();
	#endif
}
