
#include "globals.h"
#include "init.h"
#include "time.h"
#include "nip_init.h"
#include NIP_ARCH_FILE(rtl8019.h)
#include "app/dhcp.h"


/** Initialize Networking
 *
 * @return
 *    - \ref SUCCESS if initialization was successfull
 *    - \ref ERROR if initialization failed
 **/
success_t init_networking()
{
	nip_net_if_t    *net_if;

	// initialize network stack
	nip_init();

	// configure network interface

	#if (NIP_RTL_ENABLE==1 && NIP_NET_ENABLE ==1)
	rtl_net_if_id = nip_net_if_create( NIP_PHY_IF_ETH );

	if ( rtl_net_if_id == NIP_NET_NO_IF )
	{
		// network setup failed.
		return ERROR;
	}

	// configure network driver interface
	///@todo make driver init function (e.g. rtl_init_if) from the following code
	net_if = nip_net_if_ptr( rtl_net_if_id );
	net_if->phy_conf.open  = &nip_rtl_open;
	net_if->phy_conf.close = &nip_rtl_close;
	net_if->phy_conf.hard_send_init = &nip_rtl_send_init;
	net_if->phy_conf.hard_send      = &nip_rtl_send;
	net_if->phy_conf.hard_read_init = &nip_rtl_read_init;
	net_if->phy_conf.hard_read      = &nip_rtl_read;
	nip_set_if_flags( rtl_net_if_id, net_if->phy_conf.flags | NIP_PHY_IFF_BROADCAST | NIP_PHY_IFF_MULTICAST );

	// configure network interface (ip address)
// 	net_if->ip_conf.addr[0] = 10;
// 	net_if->ip_conf.addr[1] = 67;
// 	net_if->ip_conf.addr[2] = 74;
// 	net_if->ip_conf.addr[3] = 23;
//
// 	net_if->ip_conf.subnet_mask[0] = 255;
// 	net_if->ip_conf.subnet_mask[1] = 255;
// 	net_if->ip_conf.subnet_mask[2] = 0;
// 	net_if->ip_conf.subnet_mask[3] = 0;
//
// 	net_if->ip_conf.std_gateway[0] = 10;
// 	net_if->ip_conf.std_gateway[1] = 67;
// 	net_if->ip_conf.std_gateway[2] = 74;
// 	net_if->ip_conf.std_gateway[3] = 254;

// 	net_if->ip_conf.addr[0] = 192;
// 	net_if->ip_conf.addr[1] = 168;
// 	net_if->ip_conf.addr[2] = 2;
// 	net_if->ip_conf.addr[3] = 1;
//
// 	net_if->ip_conf.subnet_mask[0] = 255;
// 	net_if->ip_conf.subnet_mask[1] = 255;
// 	net_if->ip_conf.subnet_mask[2] = 255;
// 	net_if->ip_conf.subnet_mask[3] = 0;
//
// 	net_if->ip_conf.std_gateway[0] = 192;
// 	net_if->ip_conf.std_gateway[1] = 168;
// 	net_if->ip_conf.std_gateway[2] = 2;
// 	net_if->ip_conf.std_gateway[3] = 11;

	// start networking
	if ( nip_net_if_up( rtl_net_if_id ) == NIP_SUCCESS )
	{
		return SUCCESS;
	}
	else
	{
		return ERROR;
	}

	#else /* NIP_NET_ENABLE != 1 || NIP_RTL_ENABLE!=1*/
		return ERROR;
	#endif /* NIP_NET_ENABLE == 1 && NIP_RTL_ENABLE==1*/

}
