/*##############################################################################

nIP - Testapplication

File        : telnet.c

Description : Simple Telnet Application to test TCP

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#############################################################################*/


#include "telnet.h"
#include "dispatcher.h"
#include "inet.h"
#include "os_core.h"
#include "nip_error.h"
#include "net/net_if.h"
#include "net/tcp.h"
#include "net/numbers.h"
#include "app/ports.h"

#if TELNET_ENABLE == 1

nip_tcp_sock_id_t        tel_listensock;
nip_tcp_sock_id_t        tel_opensock = NIP_TCP_NO_SOCKET;
nip_tcp_sock_id_t        tel_tmp_sock = NIP_TCP_NO_SOCKET;


void telnet_start( void )
{
	struct nip_tcp_sock_addr sockaddr;

	// open tcp socket
	nip_memset( &sockaddr, 0, sizeof( struct nip_tcp_sock_addr ) );
	sockaddr.port = NIP_TCP_PORT_TELNET;
	tel_listensock = nip_tcp_socket( &sockaddr, NIP_TCP_SOCK_FLG_LISTENING );
}

void telnet_check( void )
{
	nip_mem_size_t     size;
	uint8_t            buf[10];
	uint8_t            *msg;

	// wait for incoming connection
	if ( tel_tmp_sock != NIP_TCP_NO_SOCKET )
	{
		// wait for socket to be closed
		if ( nip_tcp_read( tel_tmp_sock, buf, 10 ) == 0 && nip_error == NIP_E_EOF )
		{
			tel_tmp_sock = NIP_TCP_NO_SOCKET;
		}
	}
	else
	{
		tel_tmp_sock = nip_tcp_accept( tel_listensock );
		if ( tel_tmp_sock != NIP_TCP_NO_SOCKET )
		{
			nip_tcp_write( tel_tmp_sock, (uint8_t*)"\r\n ~'\\._]  S u P   M o F u  [_./`~\r\n\r\n>", 38 );

			if ( tel_opensock == NIP_TCP_NO_SOCKET )
			{
				tel_opensock = tel_tmp_sock;
				tel_tmp_sock = NIP_TCP_NO_SOCKET;
			}
			else
			{
				msg = (uint8_t*)"-out of connection space. SoRRy. :(\r\n";
				if ( 0 == nip_tcp_write( tel_tmp_sock, msg, 38 ) )
				{
					nip_tcp_write( tel_tmp_sock, msg, 38 );
				}
				nip_tcp_close( tel_tmp_sock );
			}
		}
	}
	if ( tel_opensock != NIP_TCP_NO_SOCKET )
	{
		// check for incoming data
		size = nip_tcp_read( tel_opensock, buf, 10 );

		if ( size == 0 && nip_error == NIP_E_EOF )
		{
			// close connectin
			nip_tcp_close( tel_opensock );
			tel_opensock = NIP_TCP_NO_SOCKET;
		}
		else if ( size > 0 )
		{
			if ( size > 4 && nip_memcmp( buf, "quit", 4 ) == 0 )
			{
				// user wants to close :(
				nip_tcp_write( tel_opensock, (uint8_t*)"ooohhhh.... :(\r\nbye\r\n", 21 );
				nip_tcp_close( tel_opensock );
			}
			else
			{
				// yay.. we received something. :)
				nip_tcp_write( tel_opensock, (uint8_t*)"YaY. I received something.\r\n>", 29 );
			}
		}
	}

}

#endif /* NIP_TELNET_ENABLE */
