/*##############################################################################

nIP - nano IP stack

File        : time.c

Description : Functions to handle time

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de

This program 7is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#############################################################################*/

#include <stdint.h>
#include "time.h"
#include "avr/interrupt.h"
#include "dispatcher.h"

/** Time in seconds since 0 hours, 0 minutes, 0 seconds, January 1st, 1970, UTC.
 * As the clock is being set it's value may be increased or decreased and is
 * therefor not usable for linear time measurement. Use the below tickcount
 * variable for that. */
uint32_t clock;

/** Miliseconds of clock with a precision of 1/256s */
uint8_t mclock;

/** Time in seconds since system start. Use this value for timeout measurement
 * as it will always be increased linearly. */
uint32_t tickcount;


#if USE_CLOCK

/** interrupt service routine for clock timer
 */
ISR (TIMER1_OVF_vect)
{

	if ( mclock == 0xFF ) {

// /*debug*/ 		LEDS = (LEDS & ~0x20) + ((LEDS + 0x20) & 0x20);

		// increment seconds
		clock++;
		tickcount++;

		// check timer for all network interfaces
#if (NIP_DISP_ENABLE==1)
		nip_disp_notify( NIP_DISP_TIMER );
		nip_disp_notify_if( 0, NIP_DISP_IF_TIMER );
// 		nip_dispatcher();
#endif
		// You can add Hooks for any modules that depend on the timer-interrupt
//		DHCP_Timer();

		// check TCP timer
// 		TCP_Check_Timer( NULL );
	}

	mclock = mclock + 1;
	TCNT1 = TCNT1 - (SYSCLK / 1024 / 256);
}

/** Initialize and start timer interrupt for clock and tickcount
 */
void Start_Clock (void)
{
	//Interrupt for the Clock enable
//	timer_enable_int(_BV(TOIE1));

	TIMSK   = _BV(TOIE1);// | _BV(TICIE);
// 	LEDCTRL=LED_MASK;

	//Setzen des Prescaler auf 1024
	TCCR1B |= (1<<CS10 | 0<<CS11 | 1<<CS12);
	return;
}

#endif //#if USE_CLOCK

